function [varargout]=fl_shrinkden_compute(varargin)

% Callback function for denoising GUI
% fl_shrinkden_compute
% Author : 
% Last Modifications : Pierrick Legrand
% February , 14th 2001
%
% 1D and 2D Wavelet Shrinkage .    
%
%
%   1.  Usage of the corresponding command:
%
%
%       [wtout] = WTShrink(wtin, trshld, mode)
%       [wtout] = WT2DShrink(wtin, trshld, mode)
%       wtin : wavelet transform
%       mode : 's' (single) or 'm' (multi) 
%
%
%   1.1.  Input Data
%
%   The input signal coulb be any highlighted  structure of the input list
%   ListBox of the main fltool  Figure: a real vector of size [M1,M2]. 
%      
%   It is selected when opening this  Figure from the corresponding
%   UiMenu of  the main  fltool  Figure, or by using the refresh PushButton.
%   When the type of the  highlighted structure does not match with the
%   authorized types, an error  message is displayed in the message 
%   StaticText of the main  fltool Figure. The name of the input
%   data is displayed in the StaticText just below.
%
%
%   2.  UIcontrols
%
%   2.1.  Control parameters
%
%
%   o  Threshold Factor : edit or slider
%      This parameter displays the value of the threshold .   
%
%
%   2.2.  Computation
%
%
%   o  Compute :  PushButton.
%      Computes the Wavelet Shrinkage of the input signal 
%      depending on the choice in the edit or slider menu.
%      It calls the  routines WTShrink.m if the input signal is a vector
%      or WT2DShrink.m if it's a matrix.
%
%   o  Help : PushButton.
%      Calls this help.
%
%   o  Close : PushButton.
%      Closes the Wavelet Shrinkage Figure and returns
%      to the main fltool Figure.
%
%   3.  Outputs
%
%   o  There is only one output ; the signal after shrinkage .
%      

switch(varargin{1})

	%% Multiplication param editing
	case 'edit_param'
      fl_clearerror;
      alpha=str2num(get(gcbo,'String'));
      if isempty(alpha)
         fl_perror('Threshold Factor must be a real !');
         pause(.3);
         alpha=1;
         set(gcbo,'String','1');
      else   
		alpha=trunc(alpha,0,5.0);
		set(gcbo,'String',alpha);
		set(findobj('Tag','Slider_shrinkden_param'),'Value',alpha);
      end;
   
	case 'slider_param'
      fl_clearerror;
      alpha=get(gcbo,'Value');
		EditHandle=findobj('Tag','EditText_shrinkden_param');
		set(EditHandle,'String',alpha);

	%% "Compute" callbacks 
      case 'compute'
	p=fl_waiton;

	%%%%% First get the input %%%%%%
	fl_clearerror;
	InputName=get(findobj('Tag','EditText_sig_nname_shrinkden'),'String');
	if isempty(InputName)
	  fl_perror('Input signal must be initiated: Refresh!');
	  fl_waitoff(p);
	else
	  eval(['global ' InputName]);
	end;


	obj=findobj('Tag','EditText_shrinkden_param');
	alpha=str2num(get(obj,'String'));
	OutputName=['den_' InputName];
	varname = fl_findname(OutputName,varargin{2});
	
	varargout{1}=varname;
	eval(['global ' varname]);
	eval(['global ' InputName]);
	% is it a vector or a matrix ?
	eval(['szmin=min(min(size(' InputName ')));']);
	if (szmin==1)
	  eval([varname '=denois(' InputName ',alpha,''visushrink'');']);
	else
	  eval([varname '=denois2D(' InputName ',alpha/10,''visushrink'');']);
	end;
	
	fl_waitoff(p);
	fl_addlist(0,varname) ;
	
   case 'refresh'
   fl_clearerror;
   [inputName flag_error]=fl_get_input('matrix');
   if flag_error
	  fl_perror('Invalid: input signal must be a matrix !');
	else
      eval(['global ' inputName]);
      %%%%%%%%%%%%%%% input frame %%%%%%%%%%%
      set(findobj('Tag','EditText_sig_nname_shrinkden'), ... 
	   'String',inputName);
   end;   
	
	case 'help'
		helpwin('fl_shrinkden_compute');



end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% trunc %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function o=trunc(i,a,b)
if(i<a)
  o=a;
else
  if(i>b)
    o=b;
  else
    o=i;
  end 
end
