function [dim,nb,taille,handlefig]=box_dimension(input_sig,Nmin,Nmax,reg,RegParam);
%   box_dimension
%   Estimates the box dimension of a 1d sample.
%   Ina Taralova
%   January 10th 2001
%
%   Computes the box dimension of the graph of a 1d sampled function.  
%   1.  Usage
%
%   o  Matlab :
%      [dim,nb,taille,handlefig]=box_dimension(input_sig,Nmin,Nmax,reg,RegParam);
%
%   1.1.  Input parameters
%
%   o  input_sig : 1d: Real vector [1,nt] 
%      Time samples of the signal to be analyzed.
%
%   o  Nmin : Integer in  [1,log2(2*floor(N/2))-1]
%      Lower size bound (lower length) of the boxes. When not
%      specified, this parameter is set to 1.
%
%   o  Nmax : Integer in  [Nmin,log2(2*floor(N/2))]
%      Upper size bound (upper length) of the boxes. When not
%      specified, this parameter is set to 5.
%
%   o  reg : Boolean
%
%      specifies weather the regression is to be done by the user or
%      automatically (default: 1). 
%
%   o  RegParam : Integer
%      specifies the type of regression to be used.
%
%   1.2.  Output parameters
%
%   o  dim : Real
%      estimated box dimension
%
%   o  nb :  Integer vector 
%      gives the number of non-empty boxes for each size of the box
%      from Nmin to Nmax
%   
%   o  taille : Real Vector
%      contains the size 2^-n of the box "n"  
%      for each "n" belonging to (Nmin, Nmax)
%
%   o  handlefig (for Matlab only): Integer vector
%      Handles of the figures opened during the procedure.
%
%   2.  See also:
%
%   cwttrack, cwtspec.
%
% This Software is ( Copyright INRIA . 2000 2001 )
% 
% INRIA  holds all the ownership rights on the Software. 
% The scientific community is asked to use the SOFTWARE 
% in order to test and evaluate it.
% 
% INRIA freely grants the right to use modify the Software,
% integrate it in another Software. 
% Any use or reproduction of this Software to obtain profit or
% for commercial purposes being subject to obtaining the prior express
% authorization of INRIA.
% 
% INRIA authorizes any reproduction of this Software.
% 
%    - in limits defined in clauses 9 and 10 of the Berne 
%    agreement for the protection of literary and artistic works 
%    respectively specify in their paragraphs 2 and 3 authorizing 
%    only the reproduction and quoting of works on the condition 
%    that :
% 
%    - "this reproduction does not adversely affect the normal 
%    exploitation of the work or cause any unjustified prejudice
%    to the legitimate interests of the author".
% 
%    - that the quotations given by way of illustration and/or 
%    tuition conform to the proper uses and that it mentions 
%    the source and name of the author if this name features 
%    in the source",
% 
%    - under the condition that this file is included with 
%    any reproduction.
%  
% Any commercial use made without obtaining the prior express 
% agreement of INRIA would therefore constitute a fraudulent
% imitation.
% 
% The Software beeing currently developed, INRIA is assuming no 
% liability, and should not be responsible, in any manner or any
% case, for any direct or indirect dammages sustained by the user.
% 
% Any user of the software shall notify at INRIA any comments 
% concerning the use of the Sofware (e-mail : FracLab@inria.fr)
% 
% This file is part of FracLab, a Fractal Analysis Software

handlefig= gcf;
clear cpt;
signal=input_sig;
[n,m]=size(signal);
if (n<m)
   signal1=signal';
   signal=signal1;
end
N=length(signal);
% Normalized signal amplitude between 0 and 1
signal1=signal*(1/(2*norm(signal,inf)));
signal1(:)=signal1(:)+0.5;

n=Nmin-1;
N=2^(floor(log2(N)));
while n<min(Nmax,N);
   n=n+1;
   cpt_lignes=zeros(2^n,1);
   c=ones(N*(2^(-n)),1);
            for j=1:2^n;
             k=0;
             cpt_colonnes=zeros(2^n,1);
             while k<2^n
                k=k+1;   
                ind1=max(1,1+round((k-1)*N*2^(-n)));
                ind2=min(N,round(k*N*(2^(-n))));
                       c=ones(ind2-ind1+1,1);
                       a=(j)*(2^(-n))*c;      
                       b=(j-1)*(2^(-n))*c;
                       a=a-signal1(ind1:ind2);
                       al=find(a<=2^(-n));
                       b=b-signal1(ind1:ind2);
                       bl=find(abs(b)<2^(-n));
                       tic
                       if (max(a(al))>0)|(min(b(bl))<=0)
                           cpt_colonnes(k)=1;
                       end; 
              end;          
             cpt_lignes(j)=sum(cpt_colonnes);              
             end;
             nb(n)=sum(cpt_lignes);
    taille(n)=2^(-n);
 end;
 if Nmin>1
    nb=nb(Nmin:n);
    taille=taille(Nmin:n);
    n=length(taille);
 end;
    %%%%%%%%%%%%%%%%
    if (reg==1)
    figure
    set(gcf,'Units','normalized','Position',[0.232 0.200 0.660 0.75])
    handlefig=[handlefig gcf];
    subplot(2,1,2)
    abc_max=log2(taille(1))+0.5;
    abc_min=log2(taille(n))-0.5;
    ord_min=log2(nb(1))-1;
    ord_max=log2(nb(n))+1;
    Vd=[abc_min abc_max ord_min ord_max]; 
    axis(Vd);
    plot(log2(taille),log2(nb),'k');hold on;
    axis(Vd);
    plot(log2(taille),log2(nb),'ko');hold off;
    title('Box Dimension')
    ylabel('log2(number)');
    xlabel('log2(size)');
    %text(-0.6,0.1,['Press return to stop'],'Color','r');
    %%%%%%%%%%%%%%%%%%
    % Message to hit "return" to end the cursor capture
    b = uicontrol('Parent',gcf,...
	'Units','normalized', ...
	'FontUnits','pixels', ...
	'FontSize',12, ...
	'FontWeight','bold', ...
   'ForegroundColor',[0.9 0 0],...
   'Position',[0.68 0.02 0.25 0.03], ...
	'String','Press return to end', ...
	'Style','text', ...
   'Tag','info');
    %%%%%%%%%%%%%%%%%%
    grid;
    lesx=get(gca,'XLim');
    %%%%%%%%%%%%%%%
    handlefig=[handlefig gcf];
    subplot(2,1,1)
    diffL=diff(log2(nb));
    diffa=diff(taille);
    newa=taille(2:length(taille));
    J=find(diffa);
    dLdarate=diffL(J)./diffa(J);
    % Sets axes margins
    abc_max=log2(taille(1))+0.5;
    abc_min=log2(taille(n))-0.5;
    ord_max1=max(diffL)+0.1*max(diffL);
    if (abs(min(diffL))>0.4)
       ord_min1=min(diffL)-0.2*min(diffL);
    else
       ord_min1=min(diffL)- 0.5;
    end;
    Vd2=[abc_min abc_max ord_min1 ord_max1]; 
    axis(Vd2);
    plot(log2(newa),diffL,'ko');
    set(gca,'XLim',lesx);
    axis(Vd2);
    title('Box Dimension')
    ylabel('Increments\Delta log2(number)');
    xlabel('log2(size)');
    grid;
           
    [X Y]=fracginput(2);
    while length(X)==2
     marqueur=0;  
     % selects indexes of elements inside the chosen range 
     % al for "Low indexes" is located to the right hand side of the graph
     al=find((log2(taille))<=max((X(1)),(X(2))));
     al=min(al);
    if (isempty(al)==1)
       title('Choose a narrow range')
       al=1;
       marqueur=1;
    end
    % ah for "High indexes" is located to the left hand side of the graph
     ah=find((log2(taille))>=min((X(1)),(X(2))));
      ah=max(ah);
      if (isempty(ah)==1)
       title('Choose a narrow range')
       ah=n;
       marqueur=1;
     end
     log(nb);
    al_f=al;
    ah_f=ah;
    if (al_f==ah_f)
       al_f=1;
       ah_f=n;
       title('Select more than one point ')
       marqueur=2;
    end
    if (al_f>ah_f)
       al_f=1;
       ah_f=n;
       title('Limits by default ')
       marqueur=1;
    end
    % Plot the selected piece of curve in red
     subplot(2,1,1)
     title('Box dimension')
     hold on;
     plot(log2(newa),diffL,'ko');
     %plot(log2(newa),diffL,'w');
     ah_f1=ah_f;
     %if (ah_f1==n)
        %ah_f1=n-1;
     %end;
     %plot(log2(newa(al_f:ah_f1-1)),diffL(al_f:ah_f1-1),'r');
     subplot(2,1,2)
     hold on;
     plot(log2(taille),log2(nb),'k');hold on;
     plot(log2(taille(al_f:ah_f)),log2(nb(al_f:ah_f)),'r');
     if strcmp(RegParam,'linf')==1|strcmp(RegParam,'lsup')==1
        %[a_hat,b_hat]=regression3(log2(taille(al_f:ah_f)),log2(nb(al_f:ah_f)));
        [pente]=newnewreg(log2(taille(al_f:ah_f)),log2(nb(al_f:ah_f)),RegParam,0);
        a_hat=pente(1);
        b_hat=pente(2);
     else
        [a_hat,b_hat]=monolr(log2(taille(al_f:ah_f)),log2(nb(al_f:ah_f)),[RegParam]);
     end;   
     dim=-mean(a_hat);
     if marqueur==0
        title(['Estimated Box Dimension = ',num2str(dim)]);
     elseif marqueur==1
        title(['Estimated Box Dimension by default = ',num2str(dim),' ,choose another range']);
     else
        title(['Estimated Box Dimension by default = ',num2str(dim),' Select more than one point']);
     end;   
     [X Y]=fracginput(2);
end;
end;
if (reg==0)
   if strcmp(RegParam,'linf')==1|strcmp(RegParam,'lsup')==1
      [pente]=newnewreg(log2(taille),log2(nb),RegParam,0);
      a_hat=pente(1);
      b_hat=pente(2);
      %[a_hat,b_hat]=regression3(log2(taille),log2(nb));
   else   
      [a_hat,b_hat]=monolr(log2(taille),log2(nb),[RegParam]);
   end;   
   dim=-mean(a_hat);
end;
   